from typing import List

import agilicus

from . import context
from .input_helpers import strip_none

from .output.table import (
    spec_column,
    format_table,
    metadata_column,
)


def list_point_of_presences(
    ctx, excludes_all_tag, excludes_any_tag, includes_all_tag, includes_any_tag, **kwargs
):
    apiclient = context.get_apiclient_from_ctx(ctx)
    excludes_all_tags = tag_list_to_tag_names(excludes_all_tag)
    excludes_any_tags = tag_list_to_tag_names(excludes_any_tag)
    includes_all_tags = tag_list_to_tag_names(includes_all_tag)
    includes_any_tags = tag_list_to_tag_names(includes_any_tag)

    return apiclient.regions_api.list_point_of_presences(
        excludes_all_tag=excludes_all_tags,
        excludes_any_tag=excludes_any_tags,
        includes_all_tag=includes_all_tags,
        includes_any_tag=includes_any_tags,
        **strip_none(kwargs),
    ).point_of_presences


def add_point_of_presence(ctx, name, tag: List[str], domain=None, **kwargs):
    apiclient = context.get_apiclient_from_ctx(ctx)

    tags = []
    if tag:
        tags = tag_list_to_tag_names(tag)

    domains = []
    if domain:
        domains = [agilicus.Domain(d) for d in domain]

    routing = agilicus.PointOfPresenceRouting(domains=domains)
    pop_spec = agilicus.PointOfPresenceSpec(
        name=agilicus.FeatureTagName(name), tags=tags, routing=routing
    )
    pop = agilicus.PointOfPresence(spec=pop_spec)
    return apiclient.regions_api.add_point_of_presence(pop)


def update_point_of_presence(
    ctx,
    pop_id,
    tag: List[str],
    domain=None,
    overwrite_tags=False,
    overwrite_domains=False,
    name=None,
    **kwargs,
):
    apiclient = context.get_apiclient_from_ctx(ctx)

    original = apiclient.regions_api.get_point_of_presence(point_of_presence_id=pop_id)

    tags = []
    if tag:
        tags = tag_list_to_tag_names(tag)

    if not overwrite_tags:
        tags.extend(original.spec.tags)
        tags = tag_list_to_tag_names(list(set(str(tag) for tag in tags)))

    original.spec.tags = tags

    domains = []
    if domain:
        domains = [agilicus.Domain(d) for d in domain]

    if not overwrite_domains:
        to_write = original.spec.routing.domains
        for domain in domains:
            if domain not in to_write:
                to_write.append(domain)
        domains = to_write

    original.spec.routing.domains = domains
    if name is not None:
        original.spec.name = name

    return apiclient.regions_api.replace_point_of_presence(
        pop_id, point_of_presence=original
    )


def show_point_of_presence(ctx, pop_id):
    apiclient = context.get_apiclient_from_ctx(ctx)
    return apiclient.regions_api.get_point_of_presence(point_of_presence_id=pop_id)


def delete_point_of_presence(ctx, pop_id):
    apiclient = context.get_apiclient_from_ctx(ctx)
    return apiclient.regions_api.delete_point_of_presence(point_of_presence_id=pop_id)


def format_point_of_presences_as_text(ctx, tags):
    columns = [
        metadata_column("id"),
        spec_column("name"),
        spec_column("tags"),
        spec_column("routing.domains", "domains"),
    ]

    return format_table(ctx, tags, columns)


def tag_list_to_tag_names(tags: List[str]) -> List[agilicus.FeatureTagName]:
    return [agilicus.FeatureTagName(tag_name) for tag_name in tags]



from unittest.mock import MagicMock

class RegionsApiMock:

    def __init__(self):
        self.mock_add_point_of_presence = MagicMock()
        self.mock_delete_point_of_presence = MagicMock()
        self.mock_get_point_of_presence = MagicMock()
        self.mock_list_point_of_presences = MagicMock()
        self.mock_replace_point_of_presence = MagicMock()

    def add_point_of_presence(self, *args, **kwargs):
        """
        This method mocks the original api RegionsApi.add_point_of_presence with MagicMock.
        """
        return self.mock_add_point_of_presence(self, *args, **kwargs)

    def delete_point_of_presence(self, *args, **kwargs):
        """
        This method mocks the original api RegionsApi.delete_point_of_presence with MagicMock.
        """
        return self.mock_delete_point_of_presence(self, *args, **kwargs)

    def get_point_of_presence(self, *args, **kwargs):
        """
        This method mocks the original api RegionsApi.get_point_of_presence with MagicMock.
        """
        return self.mock_get_point_of_presence(self, *args, **kwargs)

    def list_point_of_presences(self, *args, **kwargs):
        """
        This method mocks the original api RegionsApi.list_point_of_presences with MagicMock.
        """
        return self.mock_list_point_of_presences(self, *args, **kwargs)

    def replace_point_of_presence(self, *args, **kwargs):
        """
        This method mocks the original api RegionsApi.replace_point_of_presence with MagicMock.
        """
        return self.mock_replace_point_of_presence(self, *args, **kwargs)




import click
from ..output.table import output_entry
from . import hosts


@click.command(name="add-host")
@click.argument("hostname")
@click.option("--org-id", default=None)
@click.option("--port", default=None)
@click.option("--path", default=None)
@click.option("--label", type=str, multiple=True)
@click.pass_context
def cli_command_add_host(ctx, *args, label, **kwargs):
    result = hosts.add_host(
        ctx,
        *args,
        labels=list(label),
        **kwargs,
    )
    output_entry(ctx, result.to_dict())


@click.command(name="show-host")
@click.argument("host-id")
@click.option("--org-id", default=None)
@click.pass_context
def cli_command_show_host(ctx, **kwargs):
    result = hosts.get_host(ctx, **kwargs)
    output_entry(ctx, result.to_dict())


@click.command(name="delete-host")
@click.argument("host-id")
@click.option("--org-id", default=None)
@click.pass_context
def cli_command_delete_host(ctx, *args, **kwargs):
    hosts.delete_host(ctx, *args, **kwargs)


@click.command(name="add-host-label")
@click.argument("label", nargs=-1)
@click.option("--org-id", default=None)
@click.pass_context
def cli_command_add_host_label(ctx, label, **kwargs):
    for _label in label:
        result = hosts.add_label(ctx, label=_label, **kwargs)
        output_entry(ctx, result.to_dict())


@click.command(name="list-host-labels")
@click.option("--org-id", default=None)
@click.pass_context
def cli_command_list_host_labels(ctx, **kwargs):
    results = hosts.list_labels(ctx, **kwargs)
    print(hosts.format_labels(ctx, results))


@click.command(name="list-host-orgs")
@click.option("--org-id", default=None)
@click.pass_context
def cli_command_list_hosts_orgs(ctx, **kwargs):
    results = hosts.list_orgs(ctx, **kwargs)
    print(hosts.format_orgs(ctx, results))


@click.command(name="list-hosts")
@click.option("--org-id", default=None)
@click.pass_context
def cli_command_list_hosts(ctx, **kwargs):
    results = hosts.list_hosts(ctx, **kwargs)
    print(hosts.format_hosts_as_text(ctx, results))


@click.command(name="add-host-bundle")
@click.argument("bundle")
@click.option("--org-id", default=None)
@click.option("--label", default=None)
@click.option("--label-org-id", default=None)
@click.option("--exclude", is_flag=True, default=None)
@click.pass_context
def cli_command_add_hosts_bundle(ctx, **kwargs):
    result = hosts.add_bundle(ctx, **kwargs)
    output_entry(ctx, result.to_dict())


@click.command(name="add-host-bundle-label")
@click.argument("bundle-id")
@click.argument("label")
@click.option("--label-org-id", default=None)
@click.option("--exclude", is_flag=True, default=None)
@click.pass_context
def cli_command_add_hosts_bundle_label(ctx, **kwargs):
    result = hosts.add_bundle_label(ctx, **kwargs)
    output_entry(ctx, result.to_dict())


@click.command(name="delete-host-bundle")
@click.argument("bundle-id", nargs=-1)
@click.option("--org-id", default=None)
@click.pass_context
def cli_command_delete_hosts_bundle(ctx, bundle_id, **kwargs):
    for _bundle_id in bundle_id:
        hosts.delete_bundle(ctx, bundle_id=_bundle_id, **kwargs)


@click.command(name="list-host-bundles")
@click.option("--org-id", default=None)
@click.pass_context
def cli_command_list_hosts_bundle(ctx, **kwargs):
    results = hosts.list_bundles(ctx, **kwargs)
    print(hosts.format_bundles(ctx, results))


@click.command(name="get-host-bundle")
@click.argument("bundle-id")
@click.option("--org-id", default=None)
@click.pass_context
def cli_command_get_hosts_bundle(ctx, **kwargs):
    result = hosts.get_bundle(ctx, **kwargs)
    output_entry(ctx, result.to_dict())


all_funcs = [func for func in dir() if "cli_command_" in func]


def add_commands(cli):
    glob = globals()
    for func in all_funcs:
        cli.add_command(glob[func])



from unittest.mock import MagicMock

class WhoamiApiMock:

    def __init__(self):
        self.mock_create_whoami = MagicMock()

    def create_whoami(self, *args, **kwargs):
        """
        This method mocks the original api WhoamiApi.create_whoami with MagicMock.
        """
        return self.mock_create_whoami(self, *args, **kwargs)




"""
    Agilicus API

    Agilicus is API-first. Modern software is controlled by other software, is open, is available for you to use the way you want, securely, simply.  The OpenAPI Specification in YAML format is available on [www](https://www.agilicus.com/www/api/agilicus-openapi.yaml) for importing to other tools.  A rendered, online viewable and usable version of this specification is available at [api](https://www.agilicus.com/api). You may try the API inline directly in the web page. To do so, first obtain an Authentication Token (the simplest way is to install the Python SDK, and then run `agilicus-cli --issuer https://MYISSUER get-token`). You will need an org-id for most calls (and can obtain from `agilicus-cli --issuer https://MYISSUER list-orgs`). The `MYISSUER` will typically be `auth.MYDOMAIN`, and you will see it as you sign-in to the administrative UI.  This API releases on Bearer-Token authentication. To obtain a valid bearer token you will need to Authenticate to an Issuer with OpenID Connect (a superset of OAUTH2).  Your \"issuer\" will look like https://auth.MYDOMAIN. For example, when you signed-up, if you said \"use my own domain name\" and assigned a CNAME of cloud.example.com, then your issuer would be https://auth.cloud.example.com.  If you selected \"use an Agilicus supplied domain name\", your issuer would look like https://auth.myorg.agilicus.cloud.  For test purposes you can use our [Python SDK](https://pypi.org/project/agilicus/) and run `agilicus-cli --issuer https://auth.MYDOMAIN get-token`.  This API may be used in any language runtime that supports OpenAPI 3.0, or, you may use our [Python SDK](https://pypi.org/project/agilicus/), our [Typescript SDK](https://www.npmjs.com/package/@agilicus/angular), or our [Golang SDK](https://git.agilicus.com/pub/sdk-go).  100% of the activities in our system our API-driven, from our web-admin, through our progressive web applications, to all internals: there is nothing that is not accessible.  For more information, see [developer resources](https://www.agilicus.com/developer).   # noqa: E501

    The version of the OpenAPI document: 2024.05.30
    Contact: dev@agilicus.com
    Generated by: https://openapi-generator.tech
"""


import unittest

import agilicus_api
from agilicus_api.api.messages_api import MessagesApi  # noqa: E501


class TestMessagesApi(unittest.TestCase):
    """MessagesApi unit test stubs"""

    def setUp(self):
        self.api = MessagesApi()  # noqa: E501

    def tearDown(self):
        pass

    def test_bulk_delete_messages(self):
        """Test case for bulk_delete_messages

        Delete a list of messages and inbox items asociated to it.  # noqa: E501
        """
        pass

    def test_create_message(self):
        """Test case for create_message

        Send a message to a specific message endpoint.  # noqa: E501
        """
        pass

    def test_create_routed_message(self):
        """Test case for create_routed_message

        Send one or more messages, letting the system choose how to deliver it  # noqa: E501
        """
        pass

    def test_create_user_message(self):
        """Test case for create_user_message

        Send a message to a user on all (optionally of a type) endpoints.  # noqa: E501
        """
        pass

    def test_delete_inbox_item(self):
        """Test case for delete_inbox_item

        Delete an inbox item  # noqa: E501
        """
        pass

    def test_delete_message_endpoint(self):
        """Test case for delete_message_endpoint

        Delete a messaging endpoint  # noqa: E501
        """
        pass

    def test_get_inbox_item(self):
        """Test case for get_inbox_item

        Get an inbox item  # noqa: E501
        """
        pass

    def test_get_message_endpoint(self):
        """Test case for get_message_endpoint

        Get a message endpoint  # noqa: E501
        """
        pass

    def test_list_inbox_items(self):
        """Test case for list_inbox_items

        Get a list of inbox items items and information about the inbox  # noqa: E501
        """
        pass

    def test_list_message_endpoints(self):
        """Test case for list_message_endpoints

        List all message endpoints (all users or a single user)  # noqa: E501
        """
        pass

    def test_list_messages_config(self):
        """Test case for list_messages_config

        Get the config of the endpoint-types (e.g. public keys etc).  # noqa: E501
        """
        pass

    def test_replace_inbox_item(self):
        """Test case for replace_inbox_item

        replace an inbox item  # noqa: E501
        """
        pass

    def test_replace_message_endpoint(self):
        """Test case for replace_message_endpoint

        Update a messaging endpoint  # noqa: E501
        """
        pass

    def test_update_message_endpoint(self):
        """Test case for update_message_endpoint

        Register a messaging endpoint on a user.  # noqa: E501
        """
        pass


if __name__ == '__main__':
    unittest.main()



from . import context
from .input_helpers import get_org_from_input_or_ctx
from agilicus import input_helpers
import agilicus

from .output.table import (
    spec_column,
    status_column,
    format_table,
    metadata_column,
)


def query_connector(ctx, connector_id, **kwargs):
    token = context.get_token(ctx)
    apiclient = context.get_apiclient(ctx, token)
    org_id = get_org_from_input_or_ctx(ctx, **kwargs)

    params = {}
    params["org_id"] = org_id
    input_helpers.update_if_not_none(params, kwargs)
    query_results = apiclient.connectors_api.list_transfers(connector_id, **params)
    return query_results.connector_transfers


def format(ctx, transfers, **kwargs):
    columns = [
        metadata_column("id"),
        spec_column("org_id"),
        spec_column("src_instance_id"),
        spec_column("src_public_key"),
        spec_column("dst_instance_id"),
        spec_column("dst_public_key"),
        spec_column("transfer_type"),
        status_column("status"),
        status_column("encrypted_data_length"),
        status_column("expiry_time"),
    ]

    return format_table(ctx, transfers, columns)


def new_transfer(ctx, connector_id, **kwargs):
    token = context.get_token(ctx)
    apiclient = context.get_apiclient(ctx, token)
    org_id = get_org_from_input_or_ctx(ctx, **kwargs)

    spec = agilicus.ConnectorSecureTransferSpec(org_id=org_id)
    transfer = agilicus.ConnectorSecureTransfer(spec)
    return apiclient.connectors_api.create_transfer(connector_id, transfer)
